<?php

/**
 * tagManager
 *
 * Tag manager class
 *
 * @author Andchir <andchir@gmail.com>
 * @package tag_manager
 * @version 1.4beta2
 */

class tag_manager {
    
    public $modx = null;
    
/**
*
* @param object $this->modx
* @param array $config
*/
function __construct(modX &$modx, $config = array()){

    $this->modx =& $modx;
    $this->config = array_merge(array(
        "tplPath" => $this->modx->getOption('core_path').'components/tag_manager/elements/chunks/',
        "corePath" => $this->modx->getOption('core_path').'components/tag_manager/',
        "where" => "",
        "outerTpl" => "@FILE outerTpl.tpl",
        "innerTpl" => "@FILE innerTpl.tpl",
        "skipTV" => "",
        "numericTVs" => "",
        "priceName" => "",
        "category_tv_names" => "",
        "pl_ids_name" => "catalog_ids",
        "className" => $this->modx->getOption('tag_mgr.className',null,'modResource'),
        "packageName" => $this->modx->getOption('tag_mgr.packageName',null,''),
        "snippetProperties" => array()
    ),$config);
    
    if(empty($this->config['className'])) $this->config['className'] = 'modResource';
    
    $this->modx->addPackage('tag_manager', $this->config['corePath'].'model/');
    
    $this->config['category_tv_names'] = $this->config['category_tv_names'] ? explode(',',$this->config['category_tv_names']) : array();
    
    //Пакет класса объектов
    if($this->config['className'] != 'modResource'){
        $modelpath = $this->modx->getOption('core_path') . 'components/' . $this->config['packageName'] . '/model/';
        $added = $this->modx->addPackage($this->config['packageName'], $modelpath);
    }
    
}


/**
 * Возвращает массив данных чанка и кэширует чанк
 * 
 * @param string $source
 * @param array $properties
 * @return string
 */
public function getChunk($source, $properties = null){
    if(!$source) return array('name'=>'','snippet'=>'');
    $chunk_arr = array();
    $uniqid = uniqid();
    $_validTypes = array('@CHUNK','@FILE','@INLINE');
    $type = '@CHUNK';
    if (strpos($source, '@') === 0) {
        $endPos = strpos($source, ' ');
        if ($endPos > 2 && $endPos < 10) {
            $tt = substr($source, 0, $endPos);
            if (in_array($tt, $_validTypes)) {
                $type = $tt;
                $source = substr($source, $endPos + 1);
            }
        }
    }
    if (!is_string($type) || !in_array($type, $_validTypes)) $type = $this->modx->getOption('tplType', $properties, '@CHUNK');
    $content = false;
    switch ($type) {
        case '@FILE':
            $path = $this->modx->getOption('tplPath', $properties, $this->config['tplPath']);
            $key = $path . $source;
            if (file_exists($key)) {
                $content = file_get_contents($key);
            }
            if (!empty($content) && $content !== '0') {
                $chunk_arr = array('name'=>$key,'snippet'=>$content);
            }
        break;
        case '@INLINE':
            $chunk_arr = array('name'=>"{$type}-{$uniqid}",'snippet'=>$source);
        break;
        case '@CHUNK':
        default:
            $chunk = null;            
            $chunk = $this->modx->getObject('modChunk', array('name' => $source));
            if ($chunk) {
                $chunk_arr = $chunk->toArray();
            }
        break;
    }
    
    $chunk = $this->modx->newObject('modChunk');
    $chunk->fromArray($chunk_arr);
    $chunk->setCacheable(false);
    
    return $chunk;
}


/**
 * Парсит чанк, возвращает HTML-код
 * 
 * @param string $source
 * @param array $properties
 * @return string
 */
public function parseTpl($mainChunk, $properties = null) {
    $output = '';
    if(is_object($mainChunk)){
        $chunk = $this->modx->newObject('modChunk');
        $chunk->fromArray($mainChunk->toArray());
        $chunk->setCacheable(false);
        $output = $chunk->process($properties);
    }
    if (empty($output) && $output !== '0') {
        $prefix = $this->modx->getOption('tplPrefix', $properties, '');
        $chunk = $this->modx->newObject('modChunk');
        $chunk->setCacheable(false);
        $output = $chunk->process(array("{$prefix}output" => print_r($properties, true)), "<pre>[[+{$prefix}output]]</pre>");
    }
    return $output;
}


/**
 * Очищает строку от MODX-тегов
 *
 * @param string $string
 * @return string
 */
public function stripModxTags($string){
    if(isset($this->modx->sanitizePatterns['tags'])) $string = preg_replace($this->modx->sanitizePatterns['tags'], '', $string);
    if(isset($this->modx->sanitizePatterns['tags1'])) $string = preg_replace($this->modx->sanitizePatterns['tags1'], '', $string);
    if(isset($this->modx->sanitizePatterns['tags2'])) $string = preg_replace($this->modx->sanitizePatterns['tags2'], '', $string);
    return $string;
}


/**
 * Возвращает массив ID категорий сохраненных в БД tag_manager
 *
 */
public function getCateroryIds(){
    $output = array();
    
    $query = $this->modx->newQuery('tagManager');
    $query->where(array());
    $query->select($this->modx->getSelectColumns('tagManager','tagManager','',array('id','category')));
    $query->sortby('category','asc');
    $query->groupby('category');
    $categoryes = $this->modx->getCollection('tagManager',$query);
    
    if(count($categoryes)>0){
        foreach($categoryes as $category){
            array_push($output,$category->get('category'));
        }
    }
    
    return $output;
}


/**
 * Создаёт HTML-код блоков с фильтрами
 * 
 */
public function getFiltersOutput(){
    
    $output = '';
    
    $this->config = array_merge($this->config,$this->config['snippetProperties']);
    
    $category_id = intval($this->modx->getOption('categoryId', $this->config, 0));
    if(!$category_id){
        if(isset($this->config['parents']) && is_numeric($this->config['parents']))
            $category_id = $this->config['parents'];
        else
            $category_id = $this->modx->resource->get('id');
    }
    
    $current_url = $this->modx->makeUrl($this->modx->resource->get('id'));
    $js_map = intval($this->modx->getOption('jsMap', $this->config, false));
    
    $c_ids = $this->getCateroryIds();
    
    //Если для текущей категории нет сохраненных фильтров, ищем сохраненные из родительских
    if(in_array($category_id,$c_ids)){
        $filters_category_id = $category_id;
    }else{
        $p_ids = $this->modx->getParentIds($this->modx->resource->get('id'), 10, array('context'=>$this->modx->context->get('key')));
        $intersect = array_intersect($c_ids,$p_ids);
        if(count($intersect)>0){
            $filters_category_id = end($intersect);
        }else{
            $filters_category_id = 0;
        }
    }
    
    //Вытаскиваем значения тегов (фильтров)
    $items_q = $this->modx->newQuery('tagManager');
    $items_q->where(array(
        'active' => 1,
        'category' => $filters_category_id
    ));
    if(!empty($this->config['skipTV'])) $items_q->where(array('tvid:NOT IN' => explode(',',str_replace(' ','',$this->config['skipTV']))));
    $items_q->sortby('`index`','ASC');
    $items_r = $this->modx->getCollection('tagManager',$items_q);
    
    $outerTpl = $this->getChunk($this->config['outerTpl']);
    $innerTpl = $this->getChunk($this->config['innerTpl']);
    $numericTvs = $this->modx->getOption('tag_mgr.numericTVs',null,'');
    $numericTvs_arr = $numericTvs ? explode(',',str_replace(' ','',$numericTvs)) : array();
    
    $rows_out = '';
    
    if(count($items_r)>0){
        
        foreach ($items_r as $key => $row){
            
            $val_arr = $this->modx->fromJSON($row->get('tags'),true);
            array_push($this->config['category_tv_names'],$row->get('tvname'));
            
            if($this->config['className'] == 'modResource'){
                $is_numeric = in_array($row->get('tvid'),$numericTvs_arr);
            }else{
                $is_numeric = in_array($row->get('tvname'),$numericTvs_arr);
                
            }
            
            $chunkArrOuter = array(
                "id" => $row->get('tvid'),
                "name" => $row->get('tvname'),
                "caption" => $row->get('tvcaption'),
                "index" => $row->get('index'),
                "inner" => '',
                "min" => $is_numeric ? (count($val_arr[0])>0 ? min($val_arr[0]) : 0) : '',
                "max" => $is_numeric ? (count($val_arr[0])>0 ? max($val_arr[0]) : 0) : '',
                "isnumeric" => $is_numeric ? 1 : 0
            );
            
            if($chunkArrOuter['isnumeric'] == 0){
                foreach($val_arr[0] as $k => $v){
                    
                    if($v && $val_arr[1][$k]){
                        
                        $flt_value  = is_numeric($v) ? str_replace(',','.',strval($v)) : $v;//str_replace('&','_',$v)
                        $flt_selected = !empty($_GET['flt_'.$chunkArrOuter['name']]) && urldecode($_GET['flt_'.$chunkArrOuter['name']])==$flt_value;
                        
                        $chunkArrInner = array(
                            "name" => $chunkArrOuter['name'],
                            "caption" => $chunkArrOuter['caption'],
                            "value" => $flt_value,
                            "index" => $k,
                            "url" => $current_url,
                            $chunkArrOuter['name']."_selected" => $flt_selected ? ' selected="selected"' : '',
                            $chunkArrOuter['name']."_checked" => $flt_selected ? ' checked="checked"' : ''
                        );
                        
                        $chunkArrOuter['inner'] .= $this->parseTpl($innerTpl,$chunkArrInner);
                        
                    }
                    
                }
            }
            
            $output .= $this->parseTpl($outerTpl,$chunkArrOuter);
            
        }
        
        //Ставим плейсхолдер с именами TV, которые разрешены для текущей категории
        $this->modx->setPlaceholder('flt.tvnames',implode(',',$this->config['category_tv_names']));
        
    }
    
    if($js_map) $output .= $this->generateJSMap($category_id);
    
    return $output;
}


/**
 * Генерирует JS-карту товаров для фильтрации
 *
 * @param integer $category_id
 */
public function generateJSMap($category_id){
    
    $output = '';
    $resources = array();
    $res_ids = array();
    $numericTvs_arr = explode(',',str_replace(' ','',$this->config['numericTVs']));
    $table_prefix = $this->modx->config['table_prefix'];
    $depth = intval($this->modx->getOption('depth', $this->config, 5));
    $where = $this->modx->getOption('where', $this->config, '');
    
    $this->config['snippetProperties']['returnIDs'] = true;
    if(!empty($this->config['categoryId'])) $this->config['snippetProperties']['parents'] = $this->config['categoryId'];
    
    $res_ids_str = $this->modx->runSnippet('getProducts',$this->config['snippetProperties']);
    $res_ids = $res_ids_str ? explode(',',$res_ids_str) : array();
    
    //if(!is_array($this->config['category_tv_names']) || count($this->config['category_tv_names'])==0) return '';
    
    /*
    
    $pchildrens = $this->modx->getChildIds($category_id, $depth);
    array_push($pchildrens,$category_id);
    
    //Собирает ID ресурсов
    $sql = "
    SELECT `id` FROM `{$table_prefix}site_content`
    WHERE `parent` IN (".implode(',',$pchildrens).")
    ";
    if($where) $sql .= "AND ".$where;
    
    $stmt = $this->modx->prepare($sql);
    if ($stmt && $stmt->execute()) {
        while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
            array_push($res_ids,$row['id']);
        }
        $stmt->closeCursor();
    }
    unset($row,$stmt);
    
    */
    
    $this->modx->setPlaceholder($this->config['pl_ids_name'],implode(",",$res_ids));
    
    //Собираем TV
    if(is_array($this->config['category_tv_names']) && count($this->config['category_tv_names'])>0){
        $tv_sql = "
        SELECT DISTINCT `tv`.`id`, `tv`.`name`, `tvc`.`contentid`, IF(`tvc`.`value`!='',`tvc`.`value`,`tv`.`default_text`) as `value`
        FROM `{$table_prefix}site_tmplvars` AS `tv`
        LEFT JOIN `{$table_prefix}site_tmplvar_contentvalues` AS `tvc` ON `tvc`.`tmplvarid`=`tv`.`id`
        LEFT JOIN `{$table_prefix}site_tmplvar_access` AS `tva` ON `tva`.`tmplvarid`=`tv`.`id`
        WHERE `tvc`.`contentid` IN (".implode(',',$res_ids).") AND (ISNULL(`tva`.`documentgroup`))
        AND `tv`.`name` IN ('".implode("','",$this->config['category_tv_names'])."')
        ORDER BY `tv`.`rank`
        ";
        $stmt = $this->modx->prepare($tv_sql);
        if ($stmt && $stmt->execute()) {
            while ($row = $stmt->fetch(PDO::FETCH_ASSOC)){
                if(!isset($resources[$row['contentid']])) $resources[$row['contentid']] = array('id' => $row['contentid']);
                
                $value = in_array($row['name'],$numericTvs_arr) ? floatval($row['value']) : str_replace('&amp;','&',htmlspecialchars(trim($row['value'])));
                
                //Если это цена, пропускаем через плагин
                if($this->config['priceName'] && $this->config['priceName']==$row['name']){
                    //OnSHKgetProductPrice
                    $evtOut = $this->modx->invokeEvent('OnSHKgetProductPrice',array('purchaseArray' => array(), 'price' => $value));    
                    if(is_array($evtOut) && count($evtOut)>0){
                        $new_price = (float) str_replace(array(' ',','),array('','.'),$evtOut[0]);
                        if($new_price != $value){
                            $value = $new_price;
                        }
                    }
                }
                
                $resources[$row['contentid']][$row['name']] = $value;
            }
            $stmt->closeCursor();
        }
        
        //Создаёт JS-карту
        foreach($resources as $key => $res){
            $output .= $this->php2js($res).",\n";
        }
    }
    
    $output = "<script type=\"text/javascript\">
    var flt_data = {\"products\": [\n".(strlen($output)>2 ? substr($output,0,-2) : "")."\n]};\n
    var resource_id = ".($category_id ? $category_id : $this->modx->resource->get('id')).";
    </script>";
    
    return $output;
    
}

/**
 *
 *
 */
public function php2js($a=false){
    if (is_null($a)) return 'null';
    if ($a === false) return 'false';
    if ($a === true) return 'true';
    if (is_scalar($a)){
        if (is_float($a)){
            $a = str_replace(",", ".", strval($a));
        }
        static $jsonReplaces = array(array("\\", "/", "\n", "\t", "\r", "\b", "\f", '"'),
        array('\\\\', '\\/', '\\n', '\\t', '\\r', '\\b', '\\f', '\"'));
        return '"' . str_replace($jsonReplaces[0], $jsonReplaces[1], $a) . '"';
    }
    $isList = true;
    for ($i = 0, reset($a); $i < count($a); $i++, next($a)){
        if (key($a) !== $i){
            $isList = false;
            break;
        }
    }
    $result = array();
    if ($isList){
        foreach ($a as $v) $result[] = $this->php2js($v);
        return '[ ' . join(', ', $result) . ' ]';
    }else{
        foreach ($a as $k => $v) $result[] = $this->php2js($k).': '.$this->php2js($v);
        return '{ ' . join(', ', $result) . ' }';
    }
}

/**
 * Вытаскивает параметры снипета по набору параметров
 *
 * @param string $snippet_name
 * @param string $propertySetName
 */
public function getPrepSetProperties($snippet_name='getPage',$propertySetName=''){
    
    if(!$propertySetName){
        $propertySetName = $this->modx->getOption('tag_mgr.propertySetName',null,'filters');
        $template_id = $this->modx->resource->get('template');
        
        //Ищем имя набора параметров по ID шаблона
        if(strpos($propertySetName,'==')!==false){
            $propertySetName_arr = explode('||',$propertySetName);
            foreach($propertySetName_arr as $val){
                $temp_arr = explode('==',$val);
                $propertySetName = $temp_arr[1];
                if($temp_arr[0]==$template_id){
                    $propertySetName = $temp_arr[1];
                    break;
                }
            }
        }
    }
    
    $snippet = $this->modx->getObject('modSnippet',array('name'=>$snippet_name));
    $properties = $snippet->getProperties();
    $propSet = $this->modx->getObject('modPropertySet',array('name'=>$propertySetName));
    $propSetProperties = is_object($propSet) ? $propSet->getProperties() : array();
    
    foreach($propSetProperties as &$prop){
        $prop = str_replace(array(
            '[[*id]]',
            '[[*parent]]',
            '[[*pagetitle]]',
            '[[*longtitle]]'
        ),array(
            $this->modx->resource->get('id'),
            $this->modx->resource->get('parent'),
            $this->modx->resource->get('pagetitle'),
            $this->modx->resource->get('longtitle')
        ),$prop);
    }
    
    if(is_array($propSetProperties)) $properties = array_merge($properties,$propSetProperties);
    
    return $properties;
    
}


/**
 * Формирует массив дочерних документов-контейнеров
 *
 * @param integer $id
 * @param integer $depth
 * @param array $options
 */
public function getChildParentsIds($id= null, $depth= 10,array $options = array()) {
    $children= array ();
    if ($id !== null && intval($depth) >= 1) {
        $id= is_int($id) ? $id : intval($id);

        $context = '';
        if (!empty($options['context'])) {
            $this->modx->getContext($options['context']);
            $context = $options['context'];
        }
        $resourceMap = !empty($context) && !empty($this->modx->contexts[$context]->resourceMap) ? $this->modx->contexts[$context]->resourceMap : $this->modx->resourceMap;
        if (isset ($resourceMap["{$id}"])) {
            $child_arr = array();
            foreach($resourceMap["{$id}"] as $child){
                if(isset($resourceMap["{$child}"])){
                    array_push($child_arr,$child);
                }
            }
            unset($child);
            $children = $child_arr;
            foreach ($children as $child) {
                $processDepth = $depth - 1;
                if ($c = $this->getChildParentsIds($child,$processDepth,$options)) {
                    $children = array_merge($children, $c);
                }
            }
        }
    }
    return $children;
}


/**
 * Генерирует отступы
 *
 * @param integer $count
 */
public function getSpaces($count){
    $output = '';
    for($i=0;$i<$count;$i++){
        $output .= '&nbsp;&nbsp;&nbsp;&nbsp;';
    }
    return $output;
}


/**
 * Создаёт структуру категорий
 *
 * @param integer $id
 * @param integer $depth
 * @param string $context
 * @param array $res_arr
 * @param array $categories_id_arr
 */
public function getChildParents($id = null, $depth = 10, $context = '', $res_arr = array(), $categories_id_arr = array()) {
    $childrens = array();
    
    if (!empty($context) && $id !== null && intval($depth) >= 1) {
        
        $id = is_int($id) ? $id : intval($id);
        $this->modx->getContext($context);
        $resourceMap = !empty($this->modx->contexts[$context]->resourceMap) ? $this->modx->contexts[$context]->resourceMap : $this->modx->resourceMap;
        
        if (isset($resourceMap["{$id}"])) {
            $child_arr = array();
            foreach($resourceMap["{$id}"] as $child){
                if(isset($resourceMap["{$child}"])){
                    array_push($child_arr,$child);
                    if(isset($res_arr[$id][$child])){
                        $res = $res_arr[$id][$child];
                        array_push($childrens,array(
                            'id' => $res['id'],
                            'name' => $this->getSpaces(10-$depth).$res['pagetitle'],
                            'saved' => (in_array($res['id'],$categories_id_arr) ? '<span style="color:red;">&bull;</span>' : '')
                        ));
                        $processDepth = $depth - 1;
                        if ($c = $this->getChildParents($res['id'],$processDepth,$context,$res_arr,$categories_id_arr)) {
                            $childrens = array_merge($childrens, $c);
                        }
                    }
                }
            }
            unset($child);
        }
        
    }
    
    return $childrens;
}


/**
 * Формирует массив категорий
 *
 * @param integer $top_parent_id
 */
public function getCategoriesIds($top_parent_id){
    if(!is_array($top_parent_id)){
        $top_parent_id_arr = explode(',',str_replace(' ','',$top_parent_id));
    }else{
        $top_parent_id_arr = $top_parent_id;
    }
    $child_parents = array();
    foreach($top_parent_id_arr as $parent_id){
        if($parent_id){
            $parent = $this->modx->getObject('modResource',$parent_id);
            $parent_context = $parent->get('context_key');
        }else{
            $parent_context = $this->modx->getOption('tag_mgr.catalog_context',null,'web');
        }
        $pchils = $this->modx->getChildIds($parent_id,10,array('context'=>$parent_context));//$this->getChildParentsIds($parent_id,10,array('context'=>$parent_context));
        $child_parents = array_merge($child_parents,$pchils);
    }
    return $child_parents;
}


/**
 * Обновление списка TV и значений в БД
 *
 * @param integer $category_id
 * @param array $tv_arr
 * @param array $options
 */
public function updateTagsValues($category_id, $tv_arr=array(array(),array()), $options=array()){
    
    $item_count = count($tv_arr['values']);
    
    if(!is_array($options['child_parents']) || count($options['child_parents'])==0){
        $catalog_id = $this->modx->getOption('tag_mgr.catalog_id',null,0);
        $top_parent_id = $category_id ? $category_id : intval($catalog_id);
        $options['child_parents'] = $this->getCategoriesIds($top_parent_id);
    }
    
    $new_rows = array();
    
    //Если ресурсы
    if($this->config['className'] == 'modResource'){
        
        //Формируем массив с уникальными значениями TV
        $tv_query = $this->modx->newQuery('modTemplateVarResource');
        $tv_query->leftJoin('modTemplateVar','modTemplateVar',array("modTemplateVar.id = modTemplateVarResource.tmplvarid"));
        $tv_query->leftJoin('modResource','modResource',array("`modResource`.`id` = `modTemplateVarResource`.`contentid`"));
        $tv_query->where(
            array(
                '`modResource`.`template`:IN' => $options['template_arr'],
                '`modResource`.`parent`:IN' => $options['child_parents'],
                '`modTemplateVarResource`.`tmplvarid`:IN' => $options['tvsid_arr']
            )
        );
        $tv_query->select($this->modx->getSelectColumns('modTemplateVarResource','modTemplateVarResource','',array('id','tmplvarid','contentid','value')));
        $tv_query->select($this->modx->getSelectColumns('modTemplateVar','modTemplateVar','',array('name','caption')));
        $tvars = $this->modx->getCollection('modTemplateVarResource',$tv_query);
        
        if(count($tvars)>0){
            
            foreach ($tvars as $key => $tv){
                
                if(!in_array($tv->get('name'),$tv_arr['tv_names'])){
                    array_push($tv_arr['captions'],$tv->get('caption'));
                    array_push($tv_arr['tv_names'],$tv->get('name'));
                    array_push($tv_arr['tv_ids'],$tv->get('tmplvarid'));
                    array_push($tv_arr['active'],0);
                    array_push($new_rows,$item_count);
                    $item_count++;
                }
                $arr_index = array_search($tv->get('name'),$tv_arr['tv_names']);
                if($arr_index!==false){
                    
                    if(!is_array($tv_arr['values'][$arr_index])) $tv_arr['values'][$arr_index] = array(array(),array());
                    if($tv_arr['active'][$arr_index] && !in_array($tv->get('value'),$tv_arr['values'][$arr_index][0]) && strlen($tv->get('value'))>0){
                        $tv_vals_array = explode('||', $tv->get('value'));
                        foreach($tv_vals_array as $tv_val){
                            $tv_val = trim($tv_val, $options['guard_key']);
                            if (!in_array($tv_val, $tv_arr['values'][$arr_index][0])){
                                array_push($tv_arr['values'][$arr_index][0], $tv_val);
                                array_push($tv_arr['values'][$arr_index][1],1);
                            }
                        }
                    }
                    
                }
                
            }
            unset($key,$val);
        }
        
        $tv_diff_arr = array_diff($options['tvsid_arr'],$tv_arr['tv_ids']);
        
        if(!empty($tv_diff_arr)){
            //Дополняем ТВшками, у которых пока нет значений в БД
            $tv_query = $this->modx->newQuery('modTemplateVar');
            $tv_query->leftJoin('modTemplateVarTemplate','modTemplateVarTemplate',array("`modTemplateVarTemplate`.`tmplvarid` = `modTemplateVar`.`id`"));
            $tv_query->leftJoin('modResource','modResource',array("`modResource`.`template` = `modTemplateVarTemplate`.`templateid`"));
            $tv_query->where(
                array(
                    '`modResource`.`parent`:IN' => $options['child_parents'],
                    '`modTemplateVar`.`id`:IN' => $tv_diff_arr
                )
            );
            $tvars = $this->modx->getCollection('modTemplateVarResource',$tv_query);
            
            foreach ($tvars as $key => $tv){
                
                if(!in_array($tv->get('name'),$tv_arr['tv_names'])){
                    array_push($tv_arr['captions'],$tv->get('caption'));
                    array_push($tv_arr['tv_names'],$tv->get('name'));
                    array_push($tv_arr['tv_ids'],$tv->get('id'));
                    array_push($tv_arr['active'],0);
                    array_push($tv_arr['values'],array(array(),array()));
                    array_push($new_rows,$item_count);
                    $item_count++;
                }
                
            }
        }
        
        
    //Если товары из отдельной таблицы (migxdb)
    }else{
        
        //Подгружаем MIGX
        $this->modx->addPackage('migx', $this->modx->getOption('core_path') . 'components/migx/model/');
        
        $migx_config = $this->modx->getObject('migxConfig',array('name'=>$this->config['packageName']));
        
        if(is_object($migx_config)){
            
            //Создаем карту полей MIGXDB
            $fields_map = array();
            $migx_conf_str = $migx_config->get('formtabs');
            $migx_conf_arr = $this->modx->fromJSON($migx_conf_str);
            
            foreach($migx_conf_arr as $migx_data){
                $migx_data['fields'] = $this->modx->fromJSON($migx_data['fields']);
                foreach($migx_data['fields'] as $fields){
                    $fields_map[$fields['field']] = $fields['caption'];
                }
            }
            
            //Формируем массив с уникальными значениями
            $query = $this->modx->newQuery($this->config['className'],array('resource_id:IN'=>$options['child_parents']));
            $objects = $this->modx->getCollection($this->config['className'],$query);
            
            foreach ($objects as $key => $object){
                
                $object_arr = $object->toArray();
                
                foreach ($object_arr as $key => $value){
                    
                    if(!in_array($key,array_keys($fields_map))) continue;
                    
                    if(!in_array($key,$tv_arr['tv_names'])){
                        $caption = $fields_map[$key];
                        array_push($tv_arr['captions'],$caption);
                        array_push($tv_arr['tv_names'],$key);
                        array_push($tv_arr['tv_ids'],0);
                        array_push($tv_arr['active'],0);
                        array_push($new_rows,$item_count);
                        $item_count++;
                    }
                    
                    $arr_index = array_search($key,$tv_arr['tv_names']);
                    
                    if($arr_index!==false){
                        
                        if(!is_array($tv_arr['values'][$arr_index])) $tv_arr['values'][$arr_index] = array(array(),array());
                        
                        if($tv_arr['active'][$arr_index] && !in_array($value,$tv_arr['values'][$arr_index][0]) && strlen($value)>0){
                            
                            $tv_vals_array = explode('||', $value);
                            foreach($tv_vals_array as $tv_val){
                                $tv_val = trim($tv_val, $options['guard_key']);
                                if (!in_array($tv_val, $tv_arr['values'][$arr_index][0])){
                                    array_push($tv_arr['values'][$arr_index][0], $tv_val);
                                    array_push($tv_arr['values'][$arr_index][1],1);
                                }
                            }
                            
                        }
                        
                    }
                    
                }
                
            }
            
        }
        
    }
    
    //$tvs_count = $this->modx->getCount('modTemplateVarResource',$tv_query);
    //var_dump($tvs_count,$tv_query->toSQL());
    
    //Добавляем строки в таблицу tagManager
    if(count($new_rows)>0){
        
        foreach($new_rows as $key => $index){
            $row = $this->modx->newObject('tagManager');
            $row->set('tvid',$tv_arr['tv_ids'][$index]);
            $row->set('category',$category_id);
            $row->set('tvname',$tv_arr['tv_names'][$index]);
            $row->set('tvcaption',$tv_arr['captions'][$index]);
            $row->set('index',$index);
            $row->set('active',$tv_arr['active'][$index]);
            //$row->set('tags',$this->modx->toJSON($tv_arr['values'][$index]));
            $row->save();
        }
        unset($key,$index,$row);
        
    }
    
    //Обновляем данные в таблице tagManager
    if($options['update_cache']){
        
        foreach($tv_arr['tv_names'] as $key => $tv_name){
            $row = $this->modx->getObject('tagManager',array('tvname'=>$tv_name,'category'=>$category_id));
            if($row){
                $row->set('tags',$this->modx->toJSON($tv_arr['values'][$key]));
                $row->save();
            }
        }
        unset($key,$tv_name);
        
    }
    
    return $tv_arr;
    
}


/**
 * Создает массив с параметрами фильтрации из $_GET
 * 
 */
function getRequestParams(){
    
    $filter_arr = $_GET;
    $flt_arr = array();
    $parents = array();
    
    if(!empty($_GET)){
        foreach($filter_arr as $key => $flt_val){
            
            if(substr($key,0,2)!='f_') continue;
            
            if(is_array($flt_val)){
                foreach($flt_val as $k => $flt){
                    $flt = htmlspecialchars(rawurldecode(trim($flt)));
                    if(substr($key,2)=='parent' && is_numeric($flt)){
                        if(!empty($flt)) $parents[] = $flt;
                    }else{
                        $operator = '=';
                        if($k==='from') $operator = '>=';
                        if($k==='to') $operator = '<=';
                        if(!empty($flt)) $flt_arr[substr($key,2).':'.$operator][] = $flt;
                    }
                }
                unset($k,$flt);
            }else{
                $flt_val = htmlspecialchars(rawurldecode(trim($flt_val)));
                if(substr($key,2)=='parent' && is_numeric($flt_val)){
                    if(!empty($flt_val)) $parents[] = $flt_val;
                }else{
                    if(!empty($flt_val)) $flt_arr[substr($key,2)] = $flt_val;
                }
            }
            
        }
    }
    
    return array($flt_arr,$parents);
    
}


/**
 * Изменяет значение поля или TV для все документов с соотв. значением
 * 
 * @param array $options
 * 
 */
function replaceFieldValue($options){
    
    $output = true;
    
    //Если это ресурсы MODX
    if($this->config['className']=='modResource' && $options['tmplvarid']){
        
        //Обновляем значения всех TV для всех найденных документов
        if($options['is_leaf']){
            $tv_query = $this->modx->newQuery('modTemplateVarResource');
            $tv_query->leftJoin('modTemplateVar','modTemplateVar',array("modTemplateVar.id = modTemplateVarResource.tmplvarid"));
            $tv_query->leftJoin('modResource','modResource',array("`modResource`.`id` = `modTemplateVarResource`.`contentid`"));
            $tv_query->where(
                array(
                    '`modResource`.`template`:IN' => explode(',',str_replace(' ','',$this->modx->getOption('tag_mgr.templates',null,''))),
                    '`modTemplateVarResource`.`tmplvarid`:=' => $options['tmplvarid'],
                    '`modTemplateVarResource`.`value`:=' => $options['old_val']
                )
            );
            $tvars = $this->modx->getCollection('modTemplateVarResource',$tv_query);
            
            foreach($tvars as $key => $tvar){
                
                $tvar->set('value',$options['new_val']);
                $tvar->save();
                
            }
            unset($key,$tvar);
        }
        
    //Если это поле из отдельной таблицы MIGXBD
    }else if($this->config['className']!='modResource' && $options['field_name']){
        
        $category = $this->modx->getObject('modResource',array('id'=>$options['category_id']));
        $p_ids = $this->modx->getChildIds($options['category_id'], 2, array('context'=>$category->get('context_key')));
        array_push($p_ids,$options['category_id']);
        
        $query = $this->modx->newQuery($this->config['className']);
        $query->where(
            array(
                $options['field_name'] => $options['old_val'],
                'resource_id:IN' => $p_ids
            )
        );
        $fields = $this->modx->getCollection($this->config['className'],$query);
        
        foreach($fields as $key => $field){
            $field->set($options['field_name'],$options['new_val']);
            $field->save();
        }
        unset($key,$tvar);
        
    }
    
    //Сохраняем значение в таблице tagManager
    $item = $this->modx->getObject('tagManager', array("tvname" => $options['field_name'], "category" => $options['category_id']));
    if($item){
        
        //Обновление значения тега
        if($options['is_leaf']){
            
            $tv_values = $this->modx->fromJSON($item->get('tags'));
            
            if(isset($tv_values[0])){
                foreach($tv_values[0] as $key => &$val){
                    if($val == $options['old_val']){
                        $val = is_numeric(str_replace(',','.',$options['new_val'])) ? floatval(str_replace(',','.',$options['new_val'])) : $options['new_val'];
                        if(strlen($options['new_val'])==0){
                            unset($tv_values[0][$key]);
                            unset($tv_values[1][$key]);
                        }
                    }
                }
                $item->set('tags',$this->modx->toJSON($tv_values));
                $item->save();
            }
        
        //Обновление названия блока фильтров
        }else{
            
            $item->set('tvcaption',$options['new_val']);
            $item->save();
            
        }
        
    }
    
    return $output;
    
}


}


?>