<?php

/**
 * Get a list of tags
 *
 * @package tag_manager
 * @subpackage processors
 */

$update = $modx->getOption('update',$scriptProperties,false);
$update_cache = $modx->getOption('update_cache',$scriptProperties,false);
$category_id = intval($modx->getOption('category_id',$scriptProperties,0));
$className = $this->modx->getOption('tag_mgr.className',null,'modResource');

require_once MODX_CORE_PATH."components/tag_manager/model/tag_manager.class.php";
$tag_mgr = new tag_manager($modx, $scriptProperties);
$items = array();

//Конфигурация компонента
$tag_mgr_config = array();
foreach($modx->config as $key => $conf){
    if(substr($key,0,8)=='tag_mgr.') $tag_mgr_config[substr($key,8)] = $conf;
}

//Список ID TV
//Если в настройках системы не задан список, вытаскиваем все
if(!$tag_mgr_config['tvs']){
    $tv_query = $this->modx->newQuery('modTemplateVar');
    if($tag_mgr_config['tv_categoryid']){
        $tv_query->where(array('category:IN'=>explode(',',$tag_mgr_config['tv_categoryid'])));
    }
    $tv_query->select($this->modx->getSelectColumns('modTemplateVar','modTemplateVar','',array('id')));
    $tvars = $this->modx->getCollection('modTemplateVar',$tv_query);
    $tag_mgr_config['tvs_arr'] = array();
    if($tvars){
        foreach($tvars as $tvar){
            array_push($tag_mgr_config['tvs_arr'],$tvar->get('id'));
        }
    }
//Если в настройках системы задан список ID TV
}else{
    $tag_mgr_config['tvs_arr'] = explode(',',str_replace(' ','',$tag_mgr_config['tvs']));
}

$tag_mgr_config['template_arr'] = explode(',',str_replace(' ','',$tag_mgr_config['templates']));

if(/*count($tag_mgr_config['tvs_arr'])==0 || */!$tag_mgr_config['templates']) return array();

$tv_arr = array(
    'tv_ids' => array(),
    'captions' => array(),
    'tv_names' => array(),
    'active' => array(),
    'values' => array()
);

//Массив категорий
$top_parent_id = $category_id ? $category_id : $tag_mgr_config['catalog_id'];
$top_parent_id_arr = explode(',',str_replace(' ','',$top_parent_id));
$child_parents = $tag_mgr->getCategoriesIds($top_parent_id_arr);
$child_parents = array_merge($child_parents,$top_parent_id_arr);
$item_count = 0;

//Удаляем строки таблицы, которые не соответствуют конфигурации компонента
if($update_cache){
    
    if($className=='modResource'){
        $removed = $modx->removeCollection('tagManager',
            array("tvid:NOT IN" => $tag_mgr_config['tvs_arr'])
        );
    }
    
    //Обновляем индексы сортировки
    $items_q = $modx->newQuery('tagManager');
    $items_q->where(array('category' => $category_id));
    $items_q->sortby('`index`','ASC');
    $items_r = $modx->getCollection('tagManager',$items_q);
    if(count($items_r)>0){
        foreach ($items_r as $key => $row){
            $row->set('index',$item_count);
            $row->save();
            $item_count++;
        }
        unset($key,$row,$item_count);
    }
    
}

//Вытаскиваем значения тегов (фильтров)
$items_q = $modx->newQuery('tagManager');
$items_q->where(array('category' => $category_id));
$items_q->sortby('`index`','ASC');
$items_r = $modx->getCollection('tagManager',$items_q);

//var_dump($category_id,count($items_r)); exit;

if(count($items_r)>0){
    
    $item_count = 0;
    
    foreach ($items_r as $key => $row){
        
        $index = $row->get('index');
        
        $tv_arr['tv_ids'][$item_count] = $row->get('tvid');
        $tv_arr['captions'][$item_count] = $row->get('tvcaption');
        $tv_arr['tv_names'][$item_count] = $row->get('tvname');
        $tv_arr['active'][$item_count] = intval($row->get('active'));
        
        $val_arr = $modx->fromJSON($row->get('tags'),true);
        
        //Проверяем существуют ли ещё значения TV с сохраненными таблице tagManager данными
        if($className == 'modResource' && isset($val_arr[0]) && count($val_arr[0])>0 && $update_cache){
            
            $tv_arr_n = array(array(),array());
            $temp_tv_id = $row->get('tvid');
            
            for($i=0;$i<count($val_arr[0]);$i++){
                
                $tv_query = $modx->newQuery('modTemplateVarResource');
                $tv_query->leftJoin('modTemplateVar','modTemplateVar',array("modTemplateVar.id = modTemplateVarResource.tmplvarid"));
                $tv_query->leftJoin('modResource','modResource',array("`modResource`.`id` = `modTemplateVarResource`.`contentid`"));
                $tv_query->where(
                    array(
                        '`modResource`.`template`:IN' => $tag_mgr_config['template_arr'],
                        '`modResource`.`parent`:IN' => $child_parents,
                        '`modTemplateVarResource`.`tmplvarid`:=' => $row->get('tvid')//$tag_mgr_config['tvs_arr'],
                        //'`modTemplateVarResource`.`value`:=' => $val_arr[0][$i]
                    )
                );
                $tv_query->where(array('`modTemplateVarResource`.`value`:=' => $val_arr[0][$i]), xPDOQuery::SQL_AND, null, 1);
                $tv_query->where(
                    array(
                        '`modTemplateVarResource`.`value`:LIKE' => '%'.$tag_mgr_config['guard_key'].$val_arr[0][$i].$tag_mgr_config['guard_key'].'%'
                    ), xPDOQuery::SQL_OR, null, 1);
                $tv_query->select($modx->getSelectColumns('modTemplateVarResource','modTemplateVarResource','',array('id','tmplvarid','contentid','value')));
                $tv_query->select($modx->getSelectColumns('modTemplateVar','modTemplateVar','',array('name','caption')));
                
                $tvs_count = $modx->getCount('modTemplateVarResource',$tv_query);
                
                if($tvs_count>0){
                    $tv_vals_array = explode('||', $val_arr[0][$i]);
                    foreach($tv_vals_array as $tv_val){
                        $tv_val = trim($tv_val, $tag_mgr_config['guard_key']);
                        if (!in_array($tv_val, $tv_arr_n[0])){
                            array_push($tv_arr_n[0], $tv_val);
                            array_push($tv_arr_n[1], $val_arr[1][$i]);
                        }
                    }
                }
                
            }
            
            if(count($tv_arr_n[0]) < count($val_arr[0])){
                $row->set('tags',$modx->toJSON($tv_arr_n));
                $row->save();
            }
            
            $val_arr = $tv_arr_n;
            unset($tv_arr_n,$temp_tv_id,$tv_query);
            
        }
        
        $tv_arr['values'][$item_count] = $val_arr;
        
        $item_count++;
        
    }
    unset($key,$row,$k,$v);
    
}else{
    
    $update = true;
    
}

//var_dump($tv_arr);

//Обновление списка TV и значений в БД
if($update){
    
    $tv_arr = $tag_mgr->updateTagsValues($category_id, $tv_arr,
        array(
            'template_arr' => $tag_mgr_config['template_arr'],
            'tvsid_arr' => $tag_mgr_config['tvs_arr'],
            'guard_key' => $tag_mgr_config['guard_key'],
            'update_cache' => $update_cache,
            'child_parents' => $child_parents
        )
    );
    
    if($update_cache){
        
        //Очистка кэша сайта
        if(!empty($top_parent_id_arr[0])){
            $top_parent = $modx->getObject('modResource',$top_parent_id_arr[0]);
            $workingContext = $modx->getContext($top_parent->get('context_key'));
        }else{
            $catalog_parent_context = $tag_mgr_config['catalog_context'];
            $workingContext = $modx->getContext($catalog_parent_context);
        }
        
        $modx->cacheManager->refresh(array(
            'resource' => array('contexts' => array($workingContext->get('key'))),
        ));
        
        return $modx->error->success('');
        
    }
    
}

//echo '<pre>'; print_r($tv_arr); echo '</pre>'; exit;

//Формируем массив для дерева тегов
foreach($tv_arr['values'] as $key => $val){
    
    //Названия блоков фильтров
    $items[$key] = array(
        "id" => $tv_arr['tv_ids'][$key],
        "name" => $tv_arr['tv_names'][$key],
        "text" => $tv_arr['captions'][$key],
        "cls" => "folder",
        "expanded" => true,
        "checked" => !isset($tv_arr['active'][$key]) || $tv_arr['active'][$key] != 0,
        "children" => array()
        //"editable" => false,
        //"draggable" => true,
        //"isTarget" => false,
        //"allowDrag" => true,
        //"allowDrop" => false,
        //"allowChildren" => false,
    );
    
    if(isset($tv_arr['values'][$key][0])){
        
        //Фильтры
        if($tv_arr['active'][$key]){
            foreach($tv_arr['values'][$key][0] as $k => $v){
                $items[$key]['children'][] = array(
                    "id" => "item-{$key}-{$k}",
                    "text" => $v,
                    "leaf" => true,
                    "editable" => true,
                    //"allowDrop" => true,
                    "checked" => !isset($tv_arr['values'][$key][1][$k]) || $tv_arr['values'][$key][1][$k] != 0
                );
            }
        }
    }
    
}
unset($key,$val);

return $modx->toJSON($items);
