
/**
 * Дерево с фильтрами (тегами)
 *
 * @class tagMgr.tree.Tree
 * @extends Ext.tree.TreePanel
 * @param {Object} config An object of options.
 * @xtype tag_mgr-tree-panel
 */

tagMgr.tree.Tree = function(config) {
    config = config || {};       
    this.config = config;
    var tl,root;
    
    Ext.applyIf(config,{
        
        title: _('tag_mgr.name')
        ,xtype: 'treepanel'
        ,id: 'tag_mgr_filters_tree-cmp'
        ,layout: 'form'
        ,autoScroll: true
        ,animate: true
        ,enableDD: true
        ,enableDrag: true
        ,enableDrop: true
        //,ddAppendOnly: true
        ,loader: new Ext.tree.TreeLoader({
            dataUrl: tagMgr.config.connector_url
            ,baseParams: {action: 'gettags', category_id: tagMgr.config.category_id}
        })
        ,root: {
            nodeType: 'async'
            ,text: config.root_name || config.rootName || ''
            ,draggable: true
            ,id: config.root_id || config.rootId || 'root'
        }
        ,frame: true
        ,rootVisible: false
        ,useArrows: true
        ,tbar: [{
            xtype: 'toolbar',
            items: [{
                text: ''
                ,icon: MODx.config.manager_url+'templates/default/images/restyle/icons/arrow_down.png'
                ,cls: 'x-btn-icon'
                ,tooltip: {text: _('tree_expand')}
                ,handler: function(){
                    Ext.getCmp('tag_mgr_filters_tree-cmp').expandAll();
                }
            },'-',{
                text: ''
                ,icon: MODx.config.manager_url+'templates/default/images/restyle/icons/arrow_up.png'
                ,cls: 'x-btn-icon'
                ,tooltip: {text: _('tree_collapse')}
                ,handler: function(){
                    Ext.getCmp('tag_mgr_filters_tree-cmp').collapseAll();
                }
            }
            ,'-',{
                text: ''
                ,icon: MODx.config.manager_url+'templates/default/images/restyle/icons/refresh.png'
                ,cls: 'x-btn-icon'
                ,tooltip: {text: _('tree_refresh')}
                ,handler: this.refresh
            }
            ,'-',{
                text: ''
                ,icon: tagMgr.config.assets_url+'/img/delete.png'
                ,tooltip: _('tag_mgr.clear_category')
                ,handler: this.deleteCategory
            }
            ]
        }]
        
    });
    this.setup(config);
    this.config = config;
};
Ext.extend(tagMgr.tree.Tree,Ext.tree.TreePanel,{
    
    setup: function(config) {
        config.listeners = config.listeners || {};
        config.listeners.render = {fn:function() {
            
        },scope:this};
        tagMgr.tree.Tree.superclass.constructor.call(this,config);
        
        this.on('beforedblclick',this.beforeDblClick,this);
        this.on('beforenodedrop',this.beforeNodeDrop,this);
        this.on('afterrender',this.afterTreeRender,this);
	
        /* Absolute positionning fix  */
        //this.on('expandnode',function(){ var cnt = Ext.getCmp('modx-content'); if (cnt) { cnt.doLayout(); }
        
    }
    
    ,beforeDblClick: function(){
        return false;
    }
    
    ,beforeNodeDrop: function(dropEvent){
        //console.log(dropEvent.point,dropEvent.target,dropEvent.target);
        
        if(dropEvent.point=='append'){
            dropEvent.cancel = true;
        }
    }
    
    ,afterTreeRender: function(cmp){
        
        var root = this;
        
        //Редактирование значений
        this.treeEditor = new Ext.tree.TreeEditor(Ext.getCmp('tag_mgr_filters_tree-cmp'), {}, {
            listeners: {
                complete: function(treeEditor, new_val, old_val){
                    if(old_val != new_val){
                        
                        var category_id = Ext.getCmp('tag_mgr-parentlist-cmp').getValue();
                        var tree = Ext.getCmp('tag_mgr_filters_tree-cmp');
                        var selected = tree.getSelectionModel().getSelectedNode();
                        var is_leaf = selected.isLeaf();
                        var parentId = is_leaf ? selected.parentNode.id : selected.id;
                        var field_name = is_leaf ? selected.parentNode.attributes.name : selected.attributes.name;
			
                        tree.getEl().mask(_('loading'),'x-mask-loading');
                        
                        MODx.Ajax.request({
                            url: tagMgr.config.connector_url
                            ,params: {
                                'action': 'rename'
                                ,'tmplvarid': parentId
				,'field_name': field_name
                                ,'category_id': category_id
                                ,'is_leaf': is_leaf ? 1 : 0
                                ,'old_val': old_val
                                ,'new_val': new_val
                            }
                            ,listeners: {
                                'success': {fn:function(r) {
                                    
                                    root.refresh();
                                    Ext.getCmp('tag_mgr_filters_tree-cmp').getEl().unmask();
                                    tree.getEl().unmask();
                                    
                                },scope:this}
                                ,'failure': {fn:function(r) {
				    
                                    Ext.Msg.alert(_('error'), r.message);
                                    tree.getEl().unmask();
                                    return false;
				    
                                },scope:this}
                            }
                        });
                        
                    }
                }
            }
        });
        
    }
    
    //Сохранение состояния дерева
    ,saveTreeState: function(btn,event){
        
        this.getEl().mask(_('loading'),'x-mask-loading');
        var root = this;
        var treeData = root.getTreeState();
        var category_id = Ext.getCmp('tag_mgr-parentlist-cmp').getValue();
        tagMgr.config.category_id = category_id;
	
        MODx.Ajax.request({
            url: tagMgr.config.connector_url
            ,params: {
                data: treeData
                ,action: 'save'
                ,category_id: category_id
            }
            ,listeners: {
                'success': {fn:function(r) {
                    root.refresh();
                    Ext.getCmp('tag_mgr_filters_tree-cmp').getEl().unmask();
                },scope:this}
                ,'failure': {fn:function(r) {
                    Ext.Msg.alert(_('error'), r.message);
                    Ext.getCmp('tag_mgr_filters_tree-cmp').getEl().unmask();
                    return false;
                },scope:this}
            }
        });
        
    }
    
    //Собирает все данные дерева
    ,getTreeState: function(){
        
        var treeData = [];
        var tree = Ext.getCmp('tag_mgr_filters_tree-cmp');
        var folders = tree.root.childNodes;
        
        for (var i = 0; i < folders.length; i++) {
	    
            treeData[i] = {
                "id": folders[i].id,
		"name": folders[i].attributes.name,
                "caption": folders[i].text,
                "active": folders[i].attributes.checked,
                "values": []
            };
            var kids = folders[i].childNodes;
            
            for (var ii = 0; ii < kids.length; ii++) {
                treeData[i].values.push({"value": kids[ii].text, "active": kids[ii].attributes.checked});
            }
            
        }
	
        return Ext.encode(treeData);
        
    }
    
    //Обновление дерева
    ,refresh: function(){
        
        var tree = Ext.getCmp('tag_mgr_filters_tree-cmp');
        tree.getEl().mask(_('loading'),'x-mask-loading');
        tree.getLoader().load(tree.root);
	Ext.getCmp('tag_mgr-parentlist-cmp').getStore().reload();
        tree.getEl().unmask();
        
    }
    
    //Обновление кэша
    ,clearCache: function(){
        
        var category_id = Ext.getCmp('tag_mgr-parentlist-cmp').getValue();
        this.getEl().mask(_('loading'),'x-mask-loading');
        var root = this;
        
        MODx.Ajax.request({
            url: tagMgr.config.connector_url
            ,params: {
                action: 'gettags'
                ,update: 1
                ,update_cache: 1
                ,category_id: category_id
            }
            ,listeners: {
                'success': {fn:function(r) {
                    root.refresh();
                    root.getEl().unmask();
                },scope:this}
                ,'failure': {fn:function(r) {
                    Ext.Msg.alert(_('error'), r.message);
                    Ext.getCmp('tag_mgr_filters_tree-cmp').getEl().unmask();
                    return false;
                },scope:this}
            }
        });
        
    }
    
    //Удаление записей выбранной категории
    ,deleteCategory: function(){
        
        var category_id = Ext.getCmp('tag_mgr-parentlist-cmp').getValue();
        
        MODx.msg.confirm({
            title: _('warning')
            ,text: _('tag_mgr.clear_category')+'?'
            ,url: tagMgr.config.connector_url
            ,params: {
                action: 'clearCategory'
                ,category_id: category_id
            }
            ,listeners: {
                'success': {fn:function() {
                    
                    tagMgr.config.category_id = 0;
                    var tree = Ext.getCmp('tag_mgr_filters_tree-cmp');
                    
                    tree.getEl().mask(_('loading'),'x-mask-loading');
                    tree.loader.baseParams.category_id = 0;
                    tree.refresh();
                    
                    Ext.getCmp('tag_mgr-parentlist-cmp').getStore().reload();
                    Ext.getCmp('tag_mgr-parentlist-cmp').setValue('');
                    
                    tree.getEl().unmask();
                    
                },scope:this}
            }
        });
        
        return true;
        
    }
    
    //Изменение категории
    ,onChangeCategory: function(combo, new_value){
        
	Ext.getCmp('tag_mgr-parentlist-cmp').getStore().reload();
	tagMgr.config.category_id = new_value;
	this.loader.baseParams.category_id = tagMgr.config.category_id;
	this.refresh();
        
    }
    
});
Ext.reg('tag_mgr-tree-panel', tagMgr.tree.Tree);



